package de.web.databasesupport.tools.database;

import java.sql.*;
import java.util.*;
import java.net.URL;

/********************************************************************
  *
  *  program     : SimpleDatabase.java
  *  author      : Helge Janicke
  *  version     : 2001-03-28(001)
  *  (c)         : Helge Janicke, Niels-Peter de Witt, Karsten Wolke
  *  email       : heljanic@agent.fho-emden.de
  *              : ndewitt@gmx.de
  *              : mail@karsten-wolke.de
  *  description : Implementation of the Database interface.
  *
  *******************************************************************
  * modifications: See at the end of the document.
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
  * This software is provided AS IS, without a warranty of any
  * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATION AND
  * WARRANTIES, INCLUDING ANY IMLIED WARRANTY OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
  * HEREBY EXCLUDED. WE SHALL NOT BE LIABLE FOR ANY DAMAGES
  * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
  * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WE
  * WILL BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
  * DIRECT, INDIRECT, SPECIAL, CENSEQUENTIAL, INCIDENTIAL OR
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
  * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
  * SOFTWARE, EVEN IF WE HAVE BEEN ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGES.
  * This software is not designed or intended for use in online
  * control of aircraft, air traffic, aircraft navigation or
  * aircraft communications; or in the design, construction,
  * operation or maintenance of any nuclear facility. Licensee
  * represents and warrants that it will not use or redistribute
  * the software for such purposes.
********************************************************************/
/** Implemented Interface Database.
*/
public class SimpleDatabase implements Database {

    public final static int TABLE_CAT = 1;
    public final static int TABLE_SCHEM = 2;
    public final static int TABLE_NAME = 3;
    public final static int TABLE_TYPE = 4;
    public final static int TABLE_REMARKS = 5;

    public final static int COLUMN_NAME = 4;
    public final static int COLUMN_DATA_TYPE = 5;
    public final static int COLUMN_TYPE_NAME = 6;
    public final static int COLUMN_SIZE = 7;
    public final static int COLUMN_BUFFER_LENGTH = 8;
    public final static int COLUMN_DECIMAL_DIGITS = 9;
    public final static int COLUMN_NUM_PREC_RADIX = 10;
    public final static int COLUMN_NULLABLE = 11;
    public final static int COLUMN_REMARKS = 12;
    public final static int COLUMN_DEF = 13;
    public final static int COLUMN_SQL_DATA_TYPE = 14;
    public final static int COLUMN_SQL_DATETIME_SUB = 15;
    public final static int COLUMN_CHAR_OCTET_LENGTH = 16;
    public final static int COLUMN_ORDINAL_POSITION = 17;
    public final static int COLUMN_IS_NULLABLE = 18;

    /** The connection to the database. */
    private Connection jdbc = null;

    /** The driver name. */
    private String jdbcDriver = null;

    /** The connection name. */
    private String jdbcConnection = null;

    /** Driver for an ODBC-Database.
     * Useable for an network database.
     */
    public final static String JDATA2_0_DRIVER = "JData2_0.sql.$Driver";

    /** Driver for an JDBC-ODBC-Database like MS Access.
     * Not usable for an network Database.
     */
    public final static String JDBC_ODBC_DRIVER = "sun.jdbc.odbc.JdbcOdbcDriver";

    /** Properties to oben a connection to a database.
     *  Can contain key "user" with String value username,
     *  key "password" with String value password.
     *  Get your driver-description to get all informationen that can
     *  be put in.
     */
    private Properties p = new Properties();

    /** Constructor
     * An URL for the JDBC:ODBC-Driver can look like "jdbc:odbc:YourDB".
     * For the JData2_0_Driver use the Constructor with username and password.
     * @param jdbcDriver The driver to connect to the database.
     * @param jdbcConnection The url of the connection.
     *
     */
    public SimpleDatabase(String jdbcDriver, String jdbcConnection) {

        this.jdbcDriver = jdbcDriver;
        this.jdbcConnection = jdbcConnection;
    }

    /** Constructor
     * An URL for the JDBC:ODBC-Driver can look like "jdbc:odbc:YourDB".
     * An URL for the JData2_0 - ODBC-Driver can look like "jdbc:JDataConnect://127.0.0.1/YourDB".
     * @param jdbcDriver The driver to connect to the database.
     * @param jdbcConnection The url of the connection.
     * @param username The name of the user.
     * @param password The password of the user.
     */
    public SimpleDatabase(String jdbcDriver, String jdbcConnection, String username, String password) {

        p.clear();
        p.put("user", username);
        p.put("password", password);

        this.jdbcDriver = jdbcDriver;
        this.jdbcConnection = jdbcConnection;
    }

    /** Constructor
     * An URL for the JDBC:ODBC-Driver can look like "jdbc:odbc:YourDB".
     * An URL for the JData2_0 - ODBC-Driver can look like "jdbc:JDataConnect://127.0.0.1/YourDB".
     * @param jdbcDriver The driver to connect to the database.
     * @param jdbcConnection The url of the connection.
     * @param username The name of the user.
     * @param password The password of the user.
     */
    public SimpleDatabase(String jdbcDriver, String jdbcConnection, Properties prop) {

        this.p = (Properties) prop.clone();

        this.jdbcDriver = jdbcDriver;
        this.jdbcConnection = jdbcConnection;
    }

    /** Opens the connection to the database.
     *  Using the specified database and driver.
     *  AutoCommit is set to false.
     *  @throw SQLException Cannot set AutoCommit to false.
     *  @throw ClassNotFoundException Cannot load driver class (check classpath).
     *  @throw InstantiationException Cannot generate a new instance of the driver.
     *  @throw IllegalAccessException If the driver class or initializer is not accessable.
     */
    public void open()
    throws SQLException, ClassNotFoundException, InstantiationException, IllegalAccessException {

        Driver drv = (Driver) Class.forName(jdbcDriver).newInstance();
        jdbc = java.sql.DriverManager.getConnection(jdbcConnection, p);
        jdbc.setAutoCommit(false);
    }

    /** Closes the connection to the database.
     *  @throw SQLException Cannot close database.
     *  @throw NotConnectdException
     */
    public void close()
    throws SQLException, NotConnectedException {
      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      //System.out.println("JDBC = "+jdbc.toString());
      this.commit();
      jdbc.close();
    }

    /** Commits pending data to the database.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public void commit()
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      jdbc.commit();
    }

    /** Precompiles a SQL statement.
     *  Increases performance for frequently used SQL statements.
     *  A openened database connection is needed.
     *  @return The precompiled statement.
     *  @throw SQLException - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public PreparedStatement precompile(String statement)
    throws SQLException, NotConnectedException {

    if (jdbc == null || isClosed()) {
        throw new NotConnectedException("database closed");
    }
    return jdbc.prepareStatement(statement);
    }

    /** Executes an Update statement.
     *  A opened database connection is needed.
     *  @param statement The SQL satement to execute.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public void executeUpdate(String statement)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      Statement stat = null;
      stat = jdbc.createStatement();
      stat.executeUpdate(statement);
      this.commit();
    }

    /** Executes an Query from a SQL statement.
     *  A opened database connection is needed.
     *  @param statement The SQL satement to execute.
     *  @return The ResultSet returned from the database.
     *  @throw SQLExeption - if database access error occurs,
     *      or the send statement does not return a value.
     *      In the second case the errorcode of the SQLException is 0.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public ResultSet executeQuery(String statement)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet result = null;
      Statement stat = null;
      stat = jdbc.createStatement();
      result = stat.executeQuery(statement);
      return result;
    }

    /** Returns names of all tables within the database.
     *  @return The names of all tables.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public String[] getTablenames()
    throws SQLException, NotConnectedException {

      ResultSet result = getTableDescriptions();
      java.util.Vector names = new java.util.Vector();
      while ( result.next() ) {
          names.add(result.getString(TABLE_NAME));
      }
      result.close();
      String[] rv = new String[names.size()];
      for (int i=0;i<names.size();i++) {
        rv[i] = names.get(i).toString();
      }
      return rv;
    }

    /** Returns names of all tables within the database.
     *  @return The names of all tables.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public java.util.Vector getTablenamesVector()
    throws SQLException, NotConnectedException {

      ResultSet result = getTableDescriptions();
      java.util.Vector names = new java.util.Vector();
      while ( result.next() ) {
          names.add(result.getString(TABLE_NAME));
      }
      result.close();
      return names;
    }


    /** Returns a ResultSet containing description of a table.
     *  @param tableName The name of the table.
     *  @return The description for the table.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public ResultSet getTableDescription(String tableName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet rv = null;
      DatabaseMetaData dbMeta = jdbc.getMetaData();
      rv = dbMeta.getTables(jdbc.getCatalog(),null,tableName,null);

      return rv;
    }

    /** Returns a ResultSet containing the descriptions of all tables.
     *  @return The description of all tables.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public ResultSet getTableDescriptions()
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet rv = null;
      DatabaseMetaData dbMeta = jdbc.getMetaData();
      rv = dbMeta.getTables(jdbc.getCatalog(),null,null,null);
      return rv;
    }

    /** Returns all column names for the table.
     *  @param tableName The table name.
     *  @return All column names in the table.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public String[] getColumnNames(String tableName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet result = getColumnDescriptions(tableName);
      java.util.Vector names = new java.util.Vector();
      while ( result.next() ) {
          names.add(result.getString(COLUMN_NAME));
      }
      result.close();
      String[] rv = new String[names.size()];
      for (int i=0;i<names.size();i++) {
        rv[i] = names.get(i).toString();
      }
     return rv;
    }

    /** Returns all column names for the table.
     *  @param tableName The table name.
     *  @return All column names in the table.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public java.util.Vector getColumnNamesVector(String tableName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet result = getColumnDescriptions(tableName);
      java.util.Vector names = new java.util.Vector();
      while ( result.next() ) {
          names.add(result.getString(COLUMN_NAME));
      }
      result.close();
      return names;
    }


    /** Returns a ResultSet containing a description of the column.
     *  @param tableName The name of the table.
     *  @param columnName The name of the column.
     *  @return The description.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public ResultSet getColumnDescription(String tableName, String columnName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet rv = null;
      DatabaseMetaData dbMeta = jdbc.getMetaData();
      rv = dbMeta.getColumns(jdbc.getCatalog(),null,tableName,columnName);

      return rv;
    }

    /** Returns a ResultSet containing a description of the column.
     *  @param tableName The name of the table.
     *  @param columnIndex The index of the column must be greater 0.
     *  @return The description.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     *  @throw IndexOutOfBoundsException - if columnIndex is negativ, 0 or
     *         is greater as available in the table.
     */
    public ResultSet getColumnDescription(String tableName, int columnIndex)
    throws SQLException, NotConnectedException, IndexOutOfBoundsException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }

      if (columnIndex<1)
        throw new IndexOutOfBoundsException("columnIndex " +
           columnIndex + " < 1 is out of available range");

      ResultSet rv = null;
      ResultSet tempResult = null;
      String columnName = "";
      tempResult = this.getColumnDescriptions(tableName);
      int maxColumns = this.getColumnCount(tableName);

      if (maxColumns >= columnIndex) {
            for (int i=0; i<columnIndex; i++) tempResult.next();
            columnName = tempResult.getString(COLUMN_NAME);
            rv = this.getColumnDescription(tableName, columnName);
            tempResult.close();
      } else throw new IndexOutOfBoundsException("columnIndex " + columnIndex + " >= " + maxColumns + " is out of available range");
      return rv;
    }

    /** Returns a ResultSet containing a description of all columns in  table.
     *  @param tableName The name of the table.
     *  @return The description.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public ResultSet getColumnDescriptions(String tableName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet rv = null;
      DatabaseMetaData dbMeta = jdbc.getMetaData();
      rv = dbMeta.getColumns(jdbc.getCatalog(),null,tableName,null);

      return rv;
    }

    /** Tests if a table exists.
     *  @param tableName The name of the table.
     *  @return <tt>true</tt> if the table exists, <tt>false</tt> otherwise.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public boolean tableExists(String tableName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }
      boolean rv = false;
      ResultSet rsName = this.getTableDescriptions();
      while (rsName.next()) {
        if (rsName.getString(TABLE_NAME).equals(tableName)) rv = true;
      }
      rsName.close();
      return rv;

    }

    /** Tests if a column exists.
     *  @param tableName The name of the table.
     *  @param columnName The name of the column.
     *  @return <tt>true</tt> if the column exists in the table,
     *          <tt>false</tt> otherwise.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public boolean columnExists(String tableName, String columnName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || jdbc.isClosed()) {
          throw new NotConnectedException("database closed");
      }
      boolean rv = false;
      ResultSet result = this.getColumnDescriptions(tableName);
      while ( result.next() ) {
        if (result.getString(COLUMN_NAME).equals(columnName)) rv = true;
      }
      result.close();
      return rv;
    }

    /** Tests if a column exists.
     *  @param tableName The name of the table.
     *  @param columnIndex The index of the column.
     *  @return <tt>true</tt> if the column exists in the table,
     *          <tt>false</tt> otherwise.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public boolean columnExists(String tableName, int columnIndex)
    throws SQLException, NotConnectedException {

      if (jdbc == null || isClosed()) {
          throw new NotConnectedException("database closed");
      }

      boolean rv = false;
      if ( (this.getColumnCount(tableName)>=columnIndex)
            &&(columnIndex>0) )
        rv = true;

      return rv;
    }

    /** Tests if is opened.
     *  @return <tt>true</tt> if the database connection is opened,
     *          <tt>false</tt> otherwise.
     *  @throw SQLExeption - if database access error occurs.
     */
    public boolean isClosed()
    throws SQLException {
      if (jdbc==null) return true;
        return jdbc.isClosed();
    }

    /** Returns the column count of the table.
     *  @param tableName The name of the table.
     *  @return The column count.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public int getColumnCount(String tableName)
    throws SQLException, NotConnectedException {

      if (jdbc == null || jdbc.isClosed()) {
          throw new NotConnectedException("database closed");
      }
      int rv = 0;
      ResultSet result = null;
      result = this.getColumnDescriptions(tableName);
      while(result.next()) rv++;
      result.close();

      return rv;
    }

    /** Returns the names of the Columns in a ResultSet.
     *  @param result The ResultSet to examine.
     *  @return All column names.
     *  @throw SQLException - if database access error occurs.
     */
    public String[] getColumnNames(ResultSet result)
    throws SQLException {

        int count = 0;
        count = getColumnCount(result);
        String[] columnnames = new String[count];
        ResultSetMetaData meta = result.getMetaData();

        for (int i=0;i<count;i++) {
            columnnames[i] = meta.getColumnName(i+1);
        }
        return columnnames;
    }

    /** Returns the names of the Columns in a ResultSet.
     *  @param result The ResultSet to examine.
     *  @return All column names.
     *  @throw SQLException - if database access error occurs.
     */
    public java.util.Vector getColumnNamesVector(ResultSet result)
    throws SQLException {

        int count = 0;
        java.util.Vector columnnames = new java.util.Vector();
        count = getColumnCount(result);
        ResultSetMetaData meta = result.getMetaData();

        for (int i=0;i<count;i++) {
            columnnames.add(meta.getColumnName(i+1));
        }
        return columnnames;
    }


    /** Returns the column count of a ResultSet.
     *  @param tableName The name of the table.
     *  @return The column count.
     *  @throw SQLExeption - if database access error occurs.
     */
    public int getColumnCount(ResultSet result)
    throws SQLException {

        int rv = 0;
        ResultSetMetaData meta = result.getMetaData();
        rv = meta.getColumnCount();

        return rv;
    }

    /** Returns the table count of the database.
     *  @return The table count.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public int getTableCount()
    throws SQLException, NotConnectedException {

      if (jdbc == null || jdbc.isClosed()) {
          throw new NotConnectedException("database closed");
      }
      ResultSet result = null;
      int counter = 0;
      result = getTableDescriptions();
      while (result.next()) counter++;

      return counter;
    }

    /** Returns the connection.
     *  @return The connection.
     */
    public Connection getConnection() {
      return jdbc;
    }

    /** Returns the driver.
     *  @return The driver.
     */
    protected String getDriver() {
      return jdbcDriver;
    }

} // end of SimpleDatabase.java

/*************************************************** MODIFICATIONS **
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2001-03-28    heljanic    implementing this class.
  * -----------------------------------------------------------------
  * 2001-03-28    niewitt     implementing this class.
  * -----------------------------------------------------------------
  * 2001-03-28    niewitt     modified getColumCout(String)
  *                           modified getColumnDescription(String,int)
  * -----------------------------------------------------------------
  * 2001-03-31      niewitt  inserted Vector getColumnNamesVector(String)
  *                          inserted Vector getTableNamesVector()
  *                          inserted Vector getColumnNamesVector(Result)
  * -----------------------------------------------------------------
  * 2001-04-01      niewitt  inserted new Contstructor with properties
  *                          change open() to open using the properties
  * -----------------------------------------------------------------
  * 2001-04-01      niewitt  modified method
  *                          getColumnDescription(String, int)
  *                             Added IndexOutOfBoundsException
  *                             if columnIndex < 0
  *                          renamed executeStatement() to
  *                             executeQuery()
  *                          inserted executeUodate()
********************************************************************/






