package infosys;

/** The Dispatcher pops Task objects from the scheduler and
 *  ensures that the Task is processed.
 */
public class Dispatcher implements java.awt.event.ActionListener {

  /** Timer to control the pop operations. */
  protected javax.swing.Timer timer = null;

  /** The related Scheduler object. */
  protected Scheduler scheduler = null;

  /** A map to store relations between Task classes and Logic objects. */
  protected java.util.HashMap map = new java.util.HashMap();

  /** Constructs a new Dispatcher with a given initial delay and Scheduler.
   *  @param delay     The initial delay.
   *  @param scheduler The scheduler to pop tasks from.
   */
  public Dispatcher(int delay, Scheduler scheduler) {
    timer = new javax.swing.Timer(delay, this);
    this.scheduler = scheduler;
  }

  /** Called by the timer interrupt.
   *  Stops the timer, starts the process method from the related Logic object and
   *  restarts the timer. If there is no Logic object specified for this kind of task
   *  all TaskListeners <code>taskInterrupted</code> methods are invoked.
   *  @param e The timers ActionEvent.
   */
  public synchronized void actionPerformed(java.awt.event.ActionEvent e) {
    Task currentTask   = scheduler.getTask();
    if (currentTask == null) return;
    Logic currentLogic = (Logic) map.get(currentTask.getClass().getName());
    if (currentLogic != null) {
      currentLogic.process(currentTask);
    } else {
      Exception exc = new UnsupportedOperationException("No Logic for this Task specified");
      currentTask.fireTaskInterrupted("No logic specified for this task.", exc);
    }
  }

  /** Starts this Dispatcher. */
  public synchronized void start() {
    timer.start();
  }

  /** Stops this Dispatcher. */
  public synchronized void stop() {
    timer.stop();
  }

  /** Registers a Logic object for a given task type.
   *  @param logic The logic to process this type of task.
   *  @param task  The class of the Task object to be processed by logic.
   */
  public void registerLogicForTask(Logic logic, Class task) {
    map.put(task.getName(), logic);
  }

}// end of class