package infosys;

/** Task is an abstract class for special Task implementations.
 *  A Task can be scheduled by an Scheduler. For priority driven
 *  schedulers a priority int is included. The higher the priority
 *  the higher the int value.<br>
 *  For a Task a TaskListener can be added. This is useful when keeping
 *  track on the progress of the Task.<br>
 *  All methods of this class are synchronized.
 *  @see TaskListener
 */
public abstract class Task {

  /** The name of the Task. Typically reset by subclasses. */
  protected String name   = "unknown Task";

  /** int representing the priority. The higher the value the higher
   *  the priority.
   */
  protected int    prio   = 0;

  /** The source that generated this Task. */
  protected Object source = null;

  /** Vector storing all TaskListeners for this Task. */
  protected java.util.Vector listener = new java.util.Vector();

  /** Adds an TaskListener to this Task.
   *  @param tl The TaskListener to add.
   */
  public synchronized void addTaskListener(TaskListener tl) {
    listener.add(tl);
  }

  /** Removes a TaskListener from this Task.
   *  @param tl The TaskListener to remove.
   */
  public synchronized void removeTaskListener(TaskListener tl) {
    listener.remove(tl);
  }

  /** Fires a TaskScheduled event.
   *  @param msg The message for this event.
   */
  public synchronized void fireTaskScheduled(String msg) {
    TaskEvent te = new TaskEvent(this, msg);
    for (int i=listener.size()-1; i>=0; i--) {
      ((TaskListener) listener.get(i)).taskScheduled(te);
    }
  }

  /** Fires a TaskProcessed event.
   *  @param msg The message for this event.
   */
  public synchronized void fireTaskProcessed(String msg) {
    TaskEvent te = new TaskEvent(this, msg);
    for (int i=listener.size()-1; i>=0; i--) {
      ((TaskListener) listener.get(i)).taskProcessed(te);
    }
  }

  /** Fires a TaskTerminated event.
   *  @param msg The message for this event.
   */
  public synchronized void fireTaskTerminated(String msg) {
    TaskEvent te = new TaskEvent(this, msg);
    for (int i=listener.size()-1; i>=0; i--) {
      ((TaskListener) listener.get(i)).taskTerminated(te);
    }
  }

  /** Fires a TaskInterrupted event.
   *  @param msg The message for this event.
   *  @param exc The exception to report.
   */
  public synchronized void fireTaskInterrupted(String msg, Exception exc) {
    TaskExceptionEvent tee = new TaskExceptionEvent(this, msg, exc);
    for (int i=listener.size()-1; i>=0; i--) {
      ((TaskListener) listener.get(i)).taskInterrupted(tee);
    }
  }

}// end of class